@@ -19,7 +19,7 @@ warnings.filterwarnings('ignore', "'U' mode is deprecated",
 warnings.filterwarnings('ignore', 'The frontend.Option class .*',
                         DeprecationWarning, module='docutils.frontend')
 
-__version__ = '7.1.1'
+__version__ = '7.2.2'
 __display_version__ = __version__  # used for command line version
 
 #: Version info for better programmatic use.
@@ -30,7 +30,7 @@ __display_version__ = __version__  # used for command line version
 #:
 #: .. versionadded:: 1.2
 #:    Before version 1.2, check the string ``sphinx.__version__``.
-version_info = (7, 1, 1, 'final', 0)
+version_info = (7, 2, 2, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -40,13 +40,13 @@ if _in_development:
     import subprocess
 
     try:
-        ret = subprocess.run(
-            ['git', 'show', '-s', '--pretty=format:%h'],
+        if ret := subprocess.run(
+            ['git', 'rev-parse', '--short', 'HEAD'],
             cwd=package_dir,
             capture_output=True,
             encoding='ascii',
-        ).stdout
-        if ret:
+            errors='surrogateescape',
+        ).stdout:
             __display_version__ += '+/' + ret.strip()
         del ret
     finally:
