@@ -1,71 +1,76 @@
-r"""
-    sphinx.ext.inheritance_diagram
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+r"""Defines a docutils directive for inserting inheritance diagrams.
 
-    Defines a docutils directive for inserting inheritance diagrams.
+Provide the directive with one or more classes or modules (separated
+by whitespace).  For modules, all of the classes in that module will
+be used.
 
-    Provide the directive with one or more classes or modules (separated
-    by whitespace).  For modules, all of the classes in that module will
-    be used.
+Example::
 
-    Example::
+   Given the following classes:
 
-       Given the following classes:
+   class A: pass
+   class B(A): pass
+   class C(A): pass
+   class D(B, C): pass
+   class E(B): pass
 
-       class A: pass
-       class B(A): pass
-       class C(A): pass
-       class D(B, C): pass
-       class E(B): pass
+   .. inheritance-diagram: D E
 
-       .. inheritance-diagram: D E
+   Produces a graph like the following:
 
-       Produces a graph like the following:
+               A
+              / \
+             B   C
+            / \ /
+           E   D
 
-                   A
-                  / \
-                 B   C
-                / \ /
-               E   D
-
-    The graph is inserted as a PNG+image map into HTML and a PDF in
-    LaTeX.
-
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
+The graph is inserted as a PNG+image map into HTML and a PDF in
+LaTeX.
 """
 
+from __future__ import annotations
+
 import builtins
+import hashlib
 import inspect
 import re
+from collections.abc import Iterable, Sequence
 from importlib import import_module
-from typing import Any, Dict, Iterable, List, Tuple
-from typing import cast
+from typing import TYPE_CHECKING, Any, cast
 
 from docutils import nodes
-from docutils.nodes import Node
 from docutils.parsers.rst import directives
 
 import sphinx
 from sphinx import addnodes
-from sphinx.application import Sphinx
-from sphinx.environment import BuildEnvironment
 from sphinx.ext.graphviz import (
-    graphviz, figure_wrapper,
-    render_dot_html, render_dot_latex, render_dot_texinfo
+    figure_wrapper,
+    graphviz,
+    render_dot_html,
+    render_dot_latex,
+    render_dot_texinfo,
 )
-from sphinx.util import md5
 from sphinx.util.docutils import SphinxDirective
-from sphinx.writers.html import HTMLTranslator
-from sphinx.writers.latex import LaTeXTranslator
-from sphinx.writers.texinfo import TexinfoTranslator
 
+if TYPE_CHECKING:
+    from docutils.nodes import Node
+
+    from sphinx.application import Sphinx
+    from sphinx.environment import BuildEnvironment
+    from sphinx.util.typing import OptionSpec
+    from sphinx.writers.html import HTML5Translator
+    from sphinx.writers.latex import LaTeXTranslator
+    from sphinx.writers.texinfo import TexinfoTranslator
 
 module_sig_re = re.compile(r'''^(?:([\w.]*)\.)?  # module names
                            (\w+)  \s* $          # class/final module name
                            ''', re.VERBOSE)
 
 
+py_builtins = [obj for obj in vars(builtins).values()
+               if inspect.isclass(obj)]
+
+
 def try_import(objname: str) -> Any:
     """Import a object or module using *name* and *currentmodule*.
     *name* should be a relative name from *currentmodule* or
@@ -136,9 +141,10 @@ class InheritanceGraph:
     from all the way to the root "object", and then is able to generate a
     graphviz dot graph from them.
     """
-    def __init__(self, class_names: List[str], currmodule: str, show_builtins: bool = False,
-                 private_bases: bool = False, parts: int = 0, aliases: Dict[str, str] = None,
-                 top_classes: List[Any] = []) -> None:
+    def __init__(self, class_names: list[str], currmodule: str, show_builtins: bool = False,
+                 private_bases: bool = False, parts: int = 0,
+                 aliases: dict[str, str] | None = None, top_classes: Sequence[Any] = (),
+                 ) -> None:
         """*class_names* is a list of child classes to show bases from.
 
         If *show_builtins* is True, then Python builtins will be shown
@@ -149,19 +155,19 @@ class InheritanceGraph:
         self.class_info = self._class_info(classes, show_builtins,
                                            private_bases, parts, aliases, top_classes)
         if not self.class_info:
-            raise InheritanceException('No classes found for '
-                                       'inheritance diagram')
+            msg = 'No classes found for inheritance diagram'
+            raise InheritanceException(msg)
 
-    def _import_classes(self, class_names: List[str], currmodule: str) -> List[Any]:
+    def _import_classes(self, class_names: list[str], currmodule: str) -> list[Any]:
         """Import a list of classes."""
-        classes = []  # type: List[Any]
+        classes: list[Any] = []
         for name in class_names:
             classes.extend(import_classes(name, currmodule))
         return classes
 
-    def _class_info(self, classes: List[Any], show_builtins: bool, private_bases: bool,
-                    parts: int, aliases: Dict[str, str], top_classes: List[Any]
-                    ) -> List[Tuple[str, str, List[str], str]]:
+    def _class_info(self, classes: list[Any], show_builtins: bool, private_bases: bool,
+                    parts: int, aliases: dict[str, str] | None, top_classes: Sequence[Any],
+                    ) -> list[tuple[str, str, list[str], str]]:
         """Return name and bases for all classes that are ancestors of
         *classes*.
 
@@ -178,7 +184,6 @@ class InheritanceGraph:
         traverse to. Multiple names can be specified separated by comma.
         """
         all_classes = {}
-        py_builtins = vars(builtins).values()
 
         def recurse(cls: Any) -> None:
             if not show_builtins and cls in py_builtins:
@@ -199,7 +204,7 @@ class InheritanceGraph:
             except Exception:  # might raise AttributeError for strange classes
                 pass
 
-            baselist = []  # type: List[str]
+            baselist: list[str] = []
             all_classes[cls] = (nodename, fullname, baselist, tooltip)
 
             if fullname in top_classes:
@@ -217,9 +222,11 @@ class InheritanceGraph:
         for cls in classes:
             recurse(cls)
 
-        return list(all_classes.values())
+        return list(all_classes.values())  # type: ignore[arg-type]
 
-    def class_name(self, cls: Any, parts: int = 0, aliases: Dict[str, str] = None) -> str:
+    def class_name(
+        self, cls: Any, parts: int = 0, aliases: dict[str, str] | None = None,
+    ) -> str:
         """Given a class object, return a fully-qualified name.
 
         This works for things I've tested in matplotlib so far, but may not be
@@ -229,7 +236,7 @@ class InheritanceGraph:
         if module in ('__builtin__', 'builtins'):
             fullname = cls.__name__
         else:
-            fullname = '%s.%s' % (module, cls.__qualname__)
+            fullname = f'{module}.{cls.__qualname__}'
         if parts == 0:
             result = fullname
         else:
@@ -239,7 +246,7 @@ class InheritanceGraph:
             return aliases[result]
         return result
 
-    def get_all_class_names(self) -> List[str]:
+    def get_all_class_names(self) -> list[str]:
         """Get all of the class names involved in the graph."""
         return [fullname for (_, fullname, _, _) in self.class_info]
 
@@ -263,14 +270,17 @@ class InheritanceGraph:
         'style': '"setlinewidth(0.5)"',
     }
 
-    def _format_node_attrs(self, attrs: Dict) -> str:
+    def _format_node_attrs(self, attrs: dict[str, Any]) -> str:
         return ','.join(['%s=%s' % x for x in sorted(attrs.items())])
 
-    def _format_graph_attrs(self, attrs: Dict) -> str:
+    def _format_graph_attrs(self, attrs: dict[str, Any]) -> str:
         return ''.join(['%s=%s;\n' % x for x in sorted(attrs.items())])
 
-    def generate_dot(self, name: str, urls: Dict = {}, env: BuildEnvironment = None,
-                     graph_attrs: Dict = {}, node_attrs: Dict = {}, edge_attrs: Dict = {}
+    def generate_dot(self, name: str, urls: dict[str, str] | None = None,
+                     env: BuildEnvironment | None = None,
+                     graph_attrs: dict | None = None,
+                     node_attrs: dict | None = None,
+                     edge_attrs: dict | None = None,
                      ) -> str:
         """Generate a graphviz dot graph from the classes that were passed in
         to __init__.
@@ -282,18 +292,23 @@ class InheritanceGraph:
         *graph_attrs*, *node_attrs*, *edge_attrs* are dictionaries containing
         key/value pairs to pass on as graphviz properties.
         """
+        if urls is None:
+            urls = {}
         g_attrs = self.default_graph_attrs.copy()
         n_attrs = self.default_node_attrs.copy()
         e_attrs = self.default_edge_attrs.copy()
-        g_attrs.update(graph_attrs)
-        n_attrs.update(node_attrs)
-        e_attrs.update(edge_attrs)
+        if graph_attrs is not None:
+            g_attrs.update(graph_attrs)
+        if node_attrs is not None:
+            n_attrs.update(node_attrs)
+        if edge_attrs is not None:
+            e_attrs.update(edge_attrs)
         if env:
             g_attrs.update(env.config.inheritance_graph_attrs)
             n_attrs.update(env.config.inheritance_node_attrs)
             e_attrs.update(env.config.inheritance_edge_attrs)
 
-        res = []  # type: List[str]
+        res: list[str] = []
         res.append('digraph %s {\n' % name)
         res.append(self._format_graph_attrs(g_attrs))
 
@@ -332,14 +347,14 @@ class InheritanceDiagram(SphinxDirective):
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
-    option_spec = {
+    option_spec: OptionSpec = {
         'parts': int,
         'private-bases': directives.flag,
         'caption': directives.unchanged,
         'top-classes': directives.unchanged_required,
     }
 
-    def run(self) -> List[Node]:
+    def run(self) -> list[Node]:
         node = inheritance_diagram()
         node.document = self.state.document
         class_names = self.arguments[0].split()
@@ -356,7 +371,7 @@ class InheritanceDiagram(SphinxDirective):
         # Create a graph starting with the list of classes
         try:
             graph = InheritanceGraph(
-                class_names, self.env.ref_context.get('py:module'),
+                class_names, self.env.ref_context.get('py:module'),  # type: ignore[arg-type]
                 parts=node['parts'],
                 private_bases='private-bases' in self.options,
                 aliases=self.config.inheritance_alias,
@@ -369,8 +384,8 @@ class InheritanceDiagram(SphinxDirective):
         # references to real URLs later.  These nodes will eventually be
         # removed from the doctree after we're done with them.
         for name in graph.get_all_class_names():
-            refnodes, x = class_role(  # type: ignore
-                'class', ':class:`%s`' % name, name, 0, self.state)  # type: ignore
+            refnodes, x = class_role(  # type: ignore[call-arg,misc]
+                'class', ':class:`%s`' % name, name, 0, self.state)  # type: ignore[arg-type]
             node.extend(refnodes)
         # Store the graph object so we can use it to generate the
         # dot file later
@@ -387,10 +402,10 @@ class InheritanceDiagram(SphinxDirective):
 
 def get_graph_hash(node: inheritance_diagram) -> str:
     encoded = (node['content'] + str(node['parts'])).encode()
-    return md5(encoded).hexdigest()[-10:]
+    return hashlib.md5(encoded, usedforsecurity=False).hexdigest()[-10:]
 
 
-def html_visit_inheritance_diagram(self: HTMLTranslator, node: inheritance_diagram) -> None:
+def html_visit_inheritance_diagram(self: HTML5Translator, node: inheritance_diagram) -> None:
     """
     Output the graph for HTML.  This will insert a PNG with clickable
     image map.
@@ -407,13 +422,16 @@ def html_visit_inheritance_diagram(self: HTMLTranslator, node: inheritance_diagr
     pending_xrefs = cast(Iterable[addnodes.pending_xref], node)
     for child in pending_xrefs:
         if child.get('refuri') is not None:
-            if graphviz_output_format == 'SVG':
-                urls[child['reftitle']] = "../" + child.get('refuri')
+            # Construct the name from the URI if the reference is external via intersphinx
+            if not child.get('internal', True):
+                refname = child['refuri'].rsplit('#', 1)[-1]
             else:
-                urls[child['reftitle']] = child.get('refuri')
+                refname = child['reftitle']
+
+            urls[refname] = child.get('refuri')
         elif child.get('refid') is not None:
             if graphviz_output_format == 'SVG':
-                urls[child['reftitle']] = '../' + current_filename + '#' + child.get('refid')
+                urls[child['reftitle']] = current_filename + '#' + child.get('refid')
             else:
                 urls[child['reftitle']] = '#' + child.get('refid')
 
@@ -438,7 +456,7 @@ def latex_visit_inheritance_diagram(self: LaTeXTranslator, node: inheritance_dia
     raise nodes.SkipNode
 
 
-def texinfo_visit_inheritance_diagram(self: TexinfoTranslator, node: inheritance_diagram
+def texinfo_visit_inheritance_diagram(self: TexinfoTranslator, node: inheritance_diagram,
                                       ) -> None:
     """
     Output the graph for Texinfo.  This will insert a PNG.
@@ -458,7 +476,7 @@ def skip(self: nodes.NodeVisitor, node: inheritance_diagram) -> None:
     raise nodes.SkipNode
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.setup_extension('sphinx.ext.graphviz')
     app.add_node(
         inheritance_diagram,
